/*
 * Add date: {creation, modification, modified} and author: {name, email}
 * to metadata if not already present
 */

var nodegit = require('nodegit');
var git = require('nodegit-kit');
var path = require('path');
var moment = require('moment');

var pathToRepo = path.resolve('content');

// Returns promise for patch of single commit
var gitShow = function(repo, commitData) {
  var commit, tree, parentTree;
  return nodegit.Revparse.single(repo, commitData.commit)
    .then((obj) => {
      return nodegit.Commit.lookup(repo, obj.id());
    })
    .then((_commit) => {
      commit = _commit;
      if(commit.parentcount() < 1) return null;
      return commit.getParents(1);
    })
    .then((parents) => {
      if(parents === null) return null;
      return parents[0].getTree();
    })
    .then((_parentTree) => {
      parentTree = _parentTree;
      return commit.getTree();
    })
    .then((_tree) => {
      tree = _tree;
      return nodegit.Diff.treeToTree(repo, parentTree, tree, null);
    })
    .then((diff) => {
      return diff.patches();
    })
    .then((patches) => {
      return [
        patches,
        commitData
      ];
    });
}

var getFilesHistory = function() {
  var repo;
  return git.open(pathToRepo)
  .then(function(_repo) {
    repo = _repo;
    return git.log(repo)
  })
  .then(function(log){
    var promises = [];
    for(var i = 0; i < log.length; i++) {
      promises.push(gitShow(repo, log[i]));
    }
    return Promise.all(promises);
  })
  .then(function(diffs) {
    var filehistory = {};
    diffs.forEach(function(data) {
      var commit = data[1];
      var diff = data[0];

      diff.forEach(function(entry) {
        var p = path.normalize(entry.newFile().path());
        if(filehistory[p]===undefined) {
          filehistory[p] = [];
        }
        filehistory[p].push(commit);
      });
    });
    return filehistory;
  });
}

module.exports = function(obj) {
  return getFilesHistory()
  .then((filesHistory) => {
    obj.pages.forEach(function(article) {
      var file = path.relative(pathToRepo, article.filename);
      var commits = filesHistory[file];
      if(commits !== undefined) {
        commits.sort(function(a,b){
          return b.date - a.date;
        });
        var newestCommit = commits[0];
        var oldestCommit = commits[commits.length-1];

        if(article.metadata.date === undefined)
          article.metadata.date = {};

        if(article.metadata.date.creation === undefined)
          article.metadata.date.creation = oldestCommit.date;

        article.metadata.date.modification = newestCommit.date;
        article.metadata.date.modified = (newestCommit !== oldestCommit);

        if(article.metadata.author === undefined)
          article.metadata.author = {}
        if(article.metadata.author.name === undefined)
          article.metadata.author.name = oldestCommit.author.name
        if(article.metadata.author.email === undefined)
          article.metadata.author.email = oldestCommit.author.email

        article.commits = commits;
      }
    });
    return obj;
  })
}
