'use strict';

var moment = require('moment');
var jobs = require('./jobs');

var toURL = function(url) {
  return jobs.run('toURL', url);
}

module.exports = function(obj) {
  return new Promise(function(resolve, reject) {
    var tags = [];
    var tagPages = {};
    obj.pages.forEach(function(article) {
      if(article.metadata.tags) {
        var ntags = [];
        article.metadata.tags.forEach(function(tag) {
          tag = {
            text: tag,
            url: toURL(tag)
          }
          ntags.push(tag);
          if(tags.map(function(e) { return e.url; }).indexOf(tag.url) < 0)
            tags.push(tag)
          if(tagPages[tag.url] === undefined) tagPages[tag.url] = [];
          tagPages[tag.url].push(article)
        })
        article.metadata.tags = ntags;
      }
    });

    obj.tags = [];
    obj.redirects = [];

    var generateTagPages = function(file, pageList) {
      var pg = {};
      pg.type = 'redirect';
      pg.file = file;
      pg.content = file+'/1';
      obj.redirects.push(pg);

      pageList.sort(function (a, b) {
        var rhs = a.metadata.date.creation;
        var lhs = b.metadata.date.creation;
        return lhs > rhs ? 1 : lhs < rhs ? -1 : 0;
      })
      var c = 0;
      let p = 0;
      var tagPage;
      var prevurl;
      var prevpage;
      var firstpage;
      var lastpage;
      var tagPageList = [];
      pageList.forEach(function(page) {
        if(c === 0) {
          p++;
          if(tagPage !== undefined) {
            tagPage.metadata.nexturl = file+'/'+p;
            obj.tags.push(tagPage);
            tagPageList.push(tagPage);
          }
          tagPage = {metadata: {}};
          if(firstpage === undefined) firstpage = tagPage;
          lastpage = tagPage;

          tagPage.file = file+'/'+p;
          tagPage.type = 'tag';
          tagPage.metadata.firstpage = firstpage;
          tagPage.metadata.prevpage = prevpage;
          tagPage.metadata.prevurl = prevurl;
          tagPage.metadata.thispage = tagPage;
          tagPage.metadata.pagenumber = p;

          tagPage.content = [];
          if(prevpage !== undefined)
            prevpage.metadata.nextpage = tagPage;
          prevurl = tagPage.file;
          prevpage = tagPage;
        }
        tagPage.content.push(page);

        c++;
        c %= obj.config.linksPerPage;
      });
      obj.tags.push(tagPage);
      tagPageList.push(tagPage);
      tagPageList.forEach(function(tagPage) {
        tagPage.metadata.lastpage = lastpage;
      });
    }

    tags.forEach(function(tag) {
      generateTagPages('tag/'+tag.url, tagPages[tag.url])
    })
    resolve(obj);
  })
}
